# coding: utf-8

"""
Immich

Immich API

The version of the OpenAPI document: 2.4.1
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from uuid import UUID
from immich.client.models.asset_ids_dto import AssetIdsDto
from immich.client.models.asset_ids_response_dto import AssetIdsResponseDto
from immich.client.models.shared_link_create_dto import SharedLinkCreateDto
from immich.client.models.shared_link_edit_dto import SharedLinkEditDto
from immich.client.models.shared_link_response_dto import SharedLinkResponseDto

from immich.client.api_client import ApiClient, RequestSerialized
from immich.client.api_response import ApiResponse
from immich.client.rest import RESTResponseType


class SharedLinksApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_call
    async def add_shared_link_assets(
        self,
        id: UUID,
        asset_ids_dto: AssetIdsDto,
        key: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[AssetIdsResponseDto]:
        """Add assets to a shared link

        Add assets to a specific shared link by its ID. This endpoint is only relevant for shared link of type individual.

        :param id: (required)
        :type id: UUID
        :param asset_ids_dto: (required)
        :type asset_ids_dto: AssetIdsDto
        :param key:
        :type key: str
        :param slug:
        :type slug: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._add_shared_link_assets_serialize(
            id=id,
            asset_ids_dto=asset_ids_dto,
            key=key,
            slug=slug,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[AssetIdsResponseDto]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def add_shared_link_assets_with_http_info(
        self,
        id: UUID,
        asset_ids_dto: AssetIdsDto,
        key: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[AssetIdsResponseDto]]:
        """Add assets to a shared link

        Add assets to a specific shared link by its ID. This endpoint is only relevant for shared link of type individual.

        :param id: (required)
        :type id: UUID
        :param asset_ids_dto: (required)
        :type asset_ids_dto: AssetIdsDto
        :param key:
        :type key: str
        :param slug:
        :type slug: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._add_shared_link_assets_serialize(
            id=id,
            asset_ids_dto=asset_ids_dto,
            key=key,
            slug=slug,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[AssetIdsResponseDto]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    async def add_shared_link_assets_without_preload_content(
        self,
        id: UUID,
        asset_ids_dto: AssetIdsDto,
        key: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Add assets to a shared link

        Add assets to a specific shared link by its ID. This endpoint is only relevant for shared link of type individual.

        :param id: (required)
        :type id: UUID
        :param asset_ids_dto: (required)
        :type asset_ids_dto: AssetIdsDto
        :param key:
        :type key: str
        :param slug:
        :type slug: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._add_shared_link_assets_serialize(
            id=id,
            asset_ids_dto=asset_ids_dto,
            key=key,
            slug=slug,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[AssetIdsResponseDto]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _add_shared_link_assets_serialize(
        self,
        id,
        asset_ids_dto,
        key,
        slug,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params["id"] = id
        # process the query parameters
        if key is not None:
            _query_params.append(("key", key))

        if slug is not None:
            _query_params.append(("slug", slug))

        # process the header parameters
        # process the form parameters
        # process the body parameter
        if asset_ids_dto is not None:
            _body_params = asset_ids_dto

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params["Content-Type"] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(
                ["application/json"]
            )
            if _default_content_type is not None:
                _header_params["Content-Type"] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ["cookie", "api_key", "bearer"]

        return self.api_client.param_serialize(
            method="PUT",
            resource_path="/shared-links/{id}/assets",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    async def create_shared_link(
        self,
        shared_link_create_dto: SharedLinkCreateDto,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SharedLinkResponseDto:
        """Create a shared link

        Create a new shared link.

        :param shared_link_create_dto: (required)
        :type shared_link_create_dto: SharedLinkCreateDto
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._create_shared_link_serialize(
            shared_link_create_dto=shared_link_create_dto,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "201": "SharedLinkResponseDto",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def create_shared_link_with_http_info(
        self,
        shared_link_create_dto: SharedLinkCreateDto,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SharedLinkResponseDto]:
        """Create a shared link

        Create a new shared link.

        :param shared_link_create_dto: (required)
        :type shared_link_create_dto: SharedLinkCreateDto
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._create_shared_link_serialize(
            shared_link_create_dto=shared_link_create_dto,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "201": "SharedLinkResponseDto",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    async def create_shared_link_without_preload_content(
        self,
        shared_link_create_dto: SharedLinkCreateDto,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Create a shared link

        Create a new shared link.

        :param shared_link_create_dto: (required)
        :type shared_link_create_dto: SharedLinkCreateDto
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._create_shared_link_serialize(
            shared_link_create_dto=shared_link_create_dto,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "201": "SharedLinkResponseDto",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _create_shared_link_serialize(
        self,
        shared_link_create_dto,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if shared_link_create_dto is not None:
            _body_params = shared_link_create_dto

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params["Content-Type"] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(
                ["application/json"]
            )
            if _default_content_type is not None:
                _header_params["Content-Type"] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ["cookie", "api_key", "bearer"]

        return self.api_client.param_serialize(
            method="POST",
            resource_path="/shared-links",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    async def get_all_shared_links(
        self,
        album_id: Optional[UUID] = None,
        id: Optional[UUID] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[SharedLinkResponseDto]:
        """Retrieve all shared links

        Retrieve a list of all shared links.

        :param album_id:
        :type album_id: UUID
        :param id:
        :type id: UUID
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_all_shared_links_serialize(
            album_id=album_id,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[SharedLinkResponseDto]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def get_all_shared_links_with_http_info(
        self,
        album_id: Optional[UUID] = None,
        id: Optional[UUID] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[SharedLinkResponseDto]]:
        """Retrieve all shared links

        Retrieve a list of all shared links.

        :param album_id:
        :type album_id: UUID
        :param id:
        :type id: UUID
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_all_shared_links_serialize(
            album_id=album_id,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[SharedLinkResponseDto]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    async def get_all_shared_links_without_preload_content(
        self,
        album_id: Optional[UUID] = None,
        id: Optional[UUID] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Retrieve all shared links

        Retrieve a list of all shared links.

        :param album_id:
        :type album_id: UUID
        :param id:
        :type id: UUID
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_all_shared_links_serialize(
            album_id=album_id,
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[SharedLinkResponseDto]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _get_all_shared_links_serialize(
        self,
        album_id,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if album_id is not None:
            _query_params.append(("albumId", album_id))

        if id is not None:
            _query_params.append(("id", id))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["cookie", "api_key", "bearer"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/shared-links",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    async def get_my_shared_link(
        self,
        key: Optional[StrictStr] = None,
        password: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        token: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SharedLinkResponseDto:
        """Retrieve current shared link

        Retrieve the current shared link associated with authentication method.

        :param key:
        :type key: str
        :param password:
        :type password: str
        :param slug:
        :type slug: str
        :param token:
        :type token: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_my_shared_link_serialize(
            key=key,
            password=password,
            slug=slug,
            token=token,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "SharedLinkResponseDto",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def get_my_shared_link_with_http_info(
        self,
        key: Optional[StrictStr] = None,
        password: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        token: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SharedLinkResponseDto]:
        """Retrieve current shared link

        Retrieve the current shared link associated with authentication method.

        :param key:
        :type key: str
        :param password:
        :type password: str
        :param slug:
        :type slug: str
        :param token:
        :type token: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_my_shared_link_serialize(
            key=key,
            password=password,
            slug=slug,
            token=token,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "SharedLinkResponseDto",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    async def get_my_shared_link_without_preload_content(
        self,
        key: Optional[StrictStr] = None,
        password: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        token: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Retrieve current shared link

        Retrieve the current shared link associated with authentication method.

        :param key:
        :type key: str
        :param password:
        :type password: str
        :param slug:
        :type slug: str
        :param token:
        :type token: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_my_shared_link_serialize(
            key=key,
            password=password,
            slug=slug,
            token=token,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "SharedLinkResponseDto",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _get_my_shared_link_serialize(
        self,
        key,
        password,
        slug,
        token,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if key is not None:
            _query_params.append(("key", key))

        if password is not None:
            _query_params.append(("password", password))

        if slug is not None:
            _query_params.append(("slug", slug))

        if token is not None:
            _query_params.append(("token", token))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["cookie", "api_key", "bearer"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/shared-links/me",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    async def get_shared_link_by_id(
        self,
        id: UUID,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SharedLinkResponseDto:
        """Retrieve a shared link

        Retrieve a specific shared link by its ID.

        :param id: (required)
        :type id: UUID
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_shared_link_by_id_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "SharedLinkResponseDto",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def get_shared_link_by_id_with_http_info(
        self,
        id: UUID,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SharedLinkResponseDto]:
        """Retrieve a shared link

        Retrieve a specific shared link by its ID.

        :param id: (required)
        :type id: UUID
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_shared_link_by_id_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "SharedLinkResponseDto",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    async def get_shared_link_by_id_without_preload_content(
        self,
        id: UUID,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Retrieve a shared link

        Retrieve a specific shared link by its ID.

        :param id: (required)
        :type id: UUID
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_shared_link_by_id_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "SharedLinkResponseDto",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _get_shared_link_by_id_serialize(
        self,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params["id"] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["cookie", "api_key", "bearer"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/shared-links/{id}",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    async def remove_shared_link(
        self,
        id: UUID,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Delete a shared link

        Delete a specific shared link by its ID.

        :param id: (required)
        :type id: UUID
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._remove_shared_link_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "204": None,
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def remove_shared_link_with_http_info(
        self,
        id: UUID,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Delete a shared link

        Delete a specific shared link by its ID.

        :param id: (required)
        :type id: UUID
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._remove_shared_link_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "204": None,
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    async def remove_shared_link_without_preload_content(
        self,
        id: UUID,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Delete a shared link

        Delete a specific shared link by its ID.

        :param id: (required)
        :type id: UUID
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._remove_shared_link_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "204": None,
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _remove_shared_link_serialize(
        self,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params["id"] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # authentication setting
        _auth_settings: List[str] = ["cookie", "api_key", "bearer"]

        return self.api_client.param_serialize(
            method="DELETE",
            resource_path="/shared-links/{id}",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    async def remove_shared_link_assets(
        self,
        id: UUID,
        asset_ids_dto: AssetIdsDto,
        key: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[AssetIdsResponseDto]:
        """Remove assets from a shared link

        Remove assets from a specific shared link by its ID. This endpoint is only relevant for shared link of type individual.

        :param id: (required)
        :type id: UUID
        :param asset_ids_dto: (required)
        :type asset_ids_dto: AssetIdsDto
        :param key:
        :type key: str
        :param slug:
        :type slug: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._remove_shared_link_assets_serialize(
            id=id,
            asset_ids_dto=asset_ids_dto,
            key=key,
            slug=slug,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[AssetIdsResponseDto]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def remove_shared_link_assets_with_http_info(
        self,
        id: UUID,
        asset_ids_dto: AssetIdsDto,
        key: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[AssetIdsResponseDto]]:
        """Remove assets from a shared link

        Remove assets from a specific shared link by its ID. This endpoint is only relevant for shared link of type individual.

        :param id: (required)
        :type id: UUID
        :param asset_ids_dto: (required)
        :type asset_ids_dto: AssetIdsDto
        :param key:
        :type key: str
        :param slug:
        :type slug: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._remove_shared_link_assets_serialize(
            id=id,
            asset_ids_dto=asset_ids_dto,
            key=key,
            slug=slug,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[AssetIdsResponseDto]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    async def remove_shared_link_assets_without_preload_content(
        self,
        id: UUID,
        asset_ids_dto: AssetIdsDto,
        key: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Remove assets from a shared link

        Remove assets from a specific shared link by its ID. This endpoint is only relevant for shared link of type individual.

        :param id: (required)
        :type id: UUID
        :param asset_ids_dto: (required)
        :type asset_ids_dto: AssetIdsDto
        :param key:
        :type key: str
        :param slug:
        :type slug: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._remove_shared_link_assets_serialize(
            id=id,
            asset_ids_dto=asset_ids_dto,
            key=key,
            slug=slug,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[AssetIdsResponseDto]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _remove_shared_link_assets_serialize(
        self,
        id,
        asset_ids_dto,
        key,
        slug,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params["id"] = id
        # process the query parameters
        if key is not None:
            _query_params.append(("key", key))

        if slug is not None:
            _query_params.append(("slug", slug))

        # process the header parameters
        # process the form parameters
        # process the body parameter
        if asset_ids_dto is not None:
            _body_params = asset_ids_dto

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params["Content-Type"] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(
                ["application/json"]
            )
            if _default_content_type is not None:
                _header_params["Content-Type"] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ["cookie", "api_key", "bearer"]

        return self.api_client.param_serialize(
            method="DELETE",
            resource_path="/shared-links/{id}/assets",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    async def update_shared_link(
        self,
        id: UUID,
        shared_link_edit_dto: SharedLinkEditDto,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SharedLinkResponseDto:
        """Update a shared link

        Update an existing shared link by its ID.

        :param id: (required)
        :type id: UUID
        :param shared_link_edit_dto: (required)
        :type shared_link_edit_dto: SharedLinkEditDto
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._update_shared_link_serialize(
            id=id,
            shared_link_edit_dto=shared_link_edit_dto,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "SharedLinkResponseDto",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def update_shared_link_with_http_info(
        self,
        id: UUID,
        shared_link_edit_dto: SharedLinkEditDto,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SharedLinkResponseDto]:
        """Update a shared link

        Update an existing shared link by its ID.

        :param id: (required)
        :type id: UUID
        :param shared_link_edit_dto: (required)
        :type shared_link_edit_dto: SharedLinkEditDto
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._update_shared_link_serialize(
            id=id,
            shared_link_edit_dto=shared_link_edit_dto,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "SharedLinkResponseDto",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    async def update_shared_link_without_preload_content(
        self,
        id: UUID,
        shared_link_edit_dto: SharedLinkEditDto,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Update a shared link

        Update an existing shared link by its ID.

        :param id: (required)
        :type id: UUID
        :param shared_link_edit_dto: (required)
        :type shared_link_edit_dto: SharedLinkEditDto
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._update_shared_link_serialize(
            id=id,
            shared_link_edit_dto=shared_link_edit_dto,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "SharedLinkResponseDto",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _update_shared_link_serialize(
        self,
        id,
        shared_link_edit_dto,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params["id"] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if shared_link_edit_dto is not None:
            _body_params = shared_link_edit_dto

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params["Content-Type"] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(
                ["application/json"]
            )
            if _default_content_type is not None:
                _header_params["Content-Type"] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ["cookie", "api_key", "bearer"]

        return self.api_client.param_serialize(
            method="PATCH",
            resource_path="/shared-links/{id}",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )



# coding: utf-8

"""
Immich

Immich API

The version of the OpenAPI document: 2.4.1
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field
from typing import Any, ClassVar, Dict, List
from immich.client.models.system_config_backups_dto import SystemConfigBackupsDto
from immich.client.models.system_config_f_fmpeg_dto import SystemConfigFFmpegDto
from immich.client.models.system_config_image_dto import SystemConfigImageDto
from immich.client.models.system_config_job_dto import SystemConfigJobDto
from immich.client.models.system_config_library_dto import SystemConfigLibraryDto
from immich.client.models.system_config_logging_dto import SystemConfigLoggingDto
from immich.client.models.system_config_machine_learning_dto import (
    SystemConfigMachineLearningDto,
)
from immich.client.models.system_config_map_dto import SystemConfigMapDto
from immich.client.models.system_config_metadata_dto import SystemConfigMetadataDto
from immich.client.models.system_config_new_version_check_dto import (
    SystemConfigNewVersionCheckDto,
)
from immich.client.models.system_config_nightly_tasks_dto import (
    SystemConfigNightlyTasksDto,
)
from immich.client.models.system_config_notifications_dto import (
    SystemConfigNotificationsDto,
)
from immich.client.models.system_config_o_auth_dto import SystemConfigOAuthDto
from immich.client.models.system_config_password_login_dto import (
    SystemConfigPasswordLoginDto,
)
from immich.client.models.system_config_reverse_geocoding_dto import (
    SystemConfigReverseGeocodingDto,
)
from immich.client.models.system_config_server_dto import SystemConfigServerDto
from immich.client.models.system_config_storage_template_dto import (
    SystemConfigStorageTemplateDto,
)
from immich.client.models.system_config_templates_dto import SystemConfigTemplatesDto
from immich.client.models.system_config_theme_dto import SystemConfigThemeDto
from immich.client.models.system_config_trash_dto import SystemConfigTrashDto
from immich.client.models.system_config_user_dto import SystemConfigUserDto
from typing import Optional, Set
from typing_extensions import Self


class SystemConfigDto(BaseModel):
    """
    SystemConfigDto
    """  # noqa: E501

    backup: SystemConfigBackupsDto
    ffmpeg: SystemConfigFFmpegDto
    image: SystemConfigImageDto
    job: SystemConfigJobDto
    library: SystemConfigLibraryDto
    logging: SystemConfigLoggingDto
    machine_learning: SystemConfigMachineLearningDto = Field(alias="machineLearning")
    map: SystemConfigMapDto
    metadata: SystemConfigMetadataDto
    new_version_check: SystemConfigNewVersionCheckDto = Field(alias="newVersionCheck")
    nightly_tasks: SystemConfigNightlyTasksDto = Field(alias="nightlyTasks")
    notifications: SystemConfigNotificationsDto
    oauth: SystemConfigOAuthDto
    password_login: SystemConfigPasswordLoginDto = Field(alias="passwordLogin")
    reverse_geocoding: SystemConfigReverseGeocodingDto = Field(alias="reverseGeocoding")
    server: SystemConfigServerDto
    storage_template: SystemConfigStorageTemplateDto = Field(alias="storageTemplate")
    templates: SystemConfigTemplatesDto
    theme: SystemConfigThemeDto
    trash: SystemConfigTrashDto
    user: SystemConfigUserDto
    __properties: ClassVar[List[str]] = [
        "backup",
        "ffmpeg",
        "image",
        "job",
        "library",
        "logging",
        "machineLearning",
        "map",
        "metadata",
        "newVersionCheck",
        "nightlyTasks",
        "notifications",
        "oauth",
        "passwordLogin",
        "reverseGeocoding",
        "server",
        "storageTemplate",
        "templates",
        "theme",
        "trash",
        "user",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SystemConfigDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of backup
        if self.backup:
            _dict["backup"] = self.backup.to_dict()
        # override the default output from pydantic by calling `to_dict()` of ffmpeg
        if self.ffmpeg:
            _dict["ffmpeg"] = self.ffmpeg.to_dict()
        # override the default output from pydantic by calling `to_dict()` of image
        if self.image:
            _dict["image"] = self.image.to_dict()
        # override the default output from pydantic by calling `to_dict()` of job
        if self.job:
            _dict["job"] = self.job.to_dict()
        # override the default output from pydantic by calling `to_dict()` of library
        if self.library:
            _dict["library"] = self.library.to_dict()
        # override the default output from pydantic by calling `to_dict()` of logging
        if self.logging:
            _dict["logging"] = self.logging.to_dict()
        # override the default output from pydantic by calling `to_dict()` of machine_learning
        if self.machine_learning:
            _dict["machineLearning"] = self.machine_learning.to_dict()
        # override the default output from pydantic by calling `to_dict()` of map
        if self.map:
            _dict["map"] = self.map.to_dict()
        # override the default output from pydantic by calling `to_dict()` of metadata
        if self.metadata:
            _dict["metadata"] = self.metadata.to_dict()
        # override the default output from pydantic by calling `to_dict()` of new_version_check
        if self.new_version_check:
            _dict["newVersionCheck"] = self.new_version_check.to_dict()
        # override the default output from pydantic by calling `to_dict()` of nightly_tasks
        if self.nightly_tasks:
            _dict["nightlyTasks"] = self.nightly_tasks.to_dict()
        # override the default output from pydantic by calling `to_dict()` of notifications
        if self.notifications:
            _dict["notifications"] = self.notifications.to_dict()
        # override the default output from pydantic by calling `to_dict()` of oauth
        if self.oauth:
            _dict["oauth"] = self.oauth.to_dict()
        # override the default output from pydantic by calling `to_dict()` of password_login
        if self.password_login:
            _dict["passwordLogin"] = self.password_login.to_dict()
        # override the default output from pydantic by calling `to_dict()` of reverse_geocoding
        if self.reverse_geocoding:
            _dict["reverseGeocoding"] = self.reverse_geocoding.to_dict()
        # override the default output from pydantic by calling `to_dict()` of server
        if self.server:
            _dict["server"] = self.server.to_dict()
        # override the default output from pydantic by calling `to_dict()` of storage_template
        if self.storage_template:
            _dict["storageTemplate"] = self.storage_template.to_dict()
        # override the default output from pydantic by calling `to_dict()` of templates
        if self.templates:
            _dict["templates"] = self.templates.to_dict()
        # override the default output from pydantic by calling `to_dict()` of theme
        if self.theme:
            _dict["theme"] = self.theme.to_dict()
        # override the default output from pydantic by calling `to_dict()` of trash
        if self.trash:
            _dict["trash"] = self.trash.to_dict()
        # override the default output from pydantic by calling `to_dict()` of user
        if self.user:
            _dict["user"] = self.user.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SystemConfigDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "backup": SystemConfigBackupsDto.from_dict(obj["backup"])
                if obj.get("backup") is not None
                else None,
                "ffmpeg": SystemConfigFFmpegDto.from_dict(obj["ffmpeg"])
                if obj.get("ffmpeg") is not None
                else None,
                "image": SystemConfigImageDto.from_dict(obj["image"])
                if obj.get("image") is not None
                else None,
                "job": SystemConfigJobDto.from_dict(obj["job"])
                if obj.get("job") is not None
                else None,
                "library": SystemConfigLibraryDto.from_dict(obj["library"])
                if obj.get("library") is not None
                else None,
                "logging": SystemConfigLoggingDto.from_dict(obj["logging"])
                if obj.get("logging") is not None
                else None,
                "machineLearning": SystemConfigMachineLearningDto.from_dict(
                    obj["machineLearning"]
                )
                if obj.get("machineLearning") is not None
                else None,
                "map": SystemConfigMapDto.from_dict(obj["map"])
                if obj.get("map") is not None
                else None,
                "metadata": SystemConfigMetadataDto.from_dict(obj["metadata"])
                if obj.get("metadata") is not None
                else None,
                "newVersionCheck": SystemConfigNewVersionCheckDto.from_dict(
                    obj["newVersionCheck"]
                )
                if obj.get("newVersionCheck") is not None
                else None,
                "nightlyTasks": SystemConfigNightlyTasksDto.from_dict(
                    obj["nightlyTasks"]
                )
                if obj.get("nightlyTasks") is not None
                else None,
                "notifications": SystemConfigNotificationsDto.from_dict(
                    obj["notifications"]
                )
                if obj.get("notifications") is not None
                else None,
                "oauth": SystemConfigOAuthDto.from_dict(obj["oauth"])
                if obj.get("oauth") is not None
                else None,
                "passwordLogin": SystemConfigPasswordLoginDto.from_dict(
                    obj["passwordLogin"]
                )
                if obj.get("passwordLogin") is not None
                else None,
                "reverseGeocoding": SystemConfigReverseGeocodingDto.from_dict(
                    obj["reverseGeocoding"]
                )
                if obj.get("reverseGeocoding") is not None
                else None,
                "server": SystemConfigServerDto.from_dict(obj["server"])
                if obj.get("server") is not None
                else None,
                "storageTemplate": SystemConfigStorageTemplateDto.from_dict(
                    obj["storageTemplate"]
                )
                if obj.get("storageTemplate") is not None
                else None,
                "templates": SystemConfigTemplatesDto.from_dict(obj["templates"])
                if obj.get("templates") is not None
                else None,
                "theme": SystemConfigThemeDto.from_dict(obj["theme"])
                if obj.get("theme") is not None
                else None,
                "trash": SystemConfigTrashDto.from_dict(obj["trash"])
                if obj.get("trash") is not None
                else None,
                "user": SystemConfigUserDto.from_dict(obj["user"])
                if obj.get("user") is not None
                else None,
            }
        )
        return _obj



# coding: utf-8

"""
Immich

Immich API

The version of the OpenAPI document: 2.4.1
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from immich.client.models.asset_ids_dto import AssetIdsDto
from immich.client.models.download_info_dto import DownloadInfoDto
from immich.client.models.download_response_dto import DownloadResponseDto

from immich.client.api_client import ApiClient, RequestSerialized
from immich.client.api_response import ApiResponse
from immich.client.rest import RESTResponseType


class DownloadApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_call
    async def download_archive(
        self,
        asset_ids_dto: AssetIdsDto,
        key: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> bytearray:
        """Download asset archive

        Download a ZIP archive containing the specified assets. The assets must have been previously requested via the \"getDownloadInfo\" endpoint.

        :param asset_ids_dto: (required)
        :type asset_ids_dto: AssetIdsDto
        :param key:
        :type key: str
        :param slug:
        :type slug: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._download_archive_serialize(
            asset_ids_dto=asset_ids_dto,
            key=key,
            slug=slug,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "bytearray",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def download_archive_with_http_info(
        self,
        asset_ids_dto: AssetIdsDto,
        key: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[bytearray]:
        """Download asset archive

        Download a ZIP archive containing the specified assets. The assets must have been previously requested via the \"getDownloadInfo\" endpoint.

        :param asset_ids_dto: (required)
        :type asset_ids_dto: AssetIdsDto
        :param key:
        :type key: str
        :param slug:
        :type slug: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._download_archive_serialize(
            asset_ids_dto=asset_ids_dto,
            key=key,
            slug=slug,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "bytearray",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    async def download_archive_without_preload_content(
        self,
        asset_ids_dto: AssetIdsDto,
        key: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Download asset archive

        Download a ZIP archive containing the specified assets. The assets must have been previously requested via the \"getDownloadInfo\" endpoint.

        :param asset_ids_dto: (required)
        :type asset_ids_dto: AssetIdsDto
        :param key:
        :type key: str
        :param slug:
        :type slug: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._download_archive_serialize(
            asset_ids_dto=asset_ids_dto,
            key=key,
            slug=slug,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "bytearray",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _download_archive_serialize(
        self,
        asset_ids_dto,
        key,
        slug,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if key is not None:
            _query_params.append(("key", key))

        if slug is not None:
            _query_params.append(("slug", slug))

        # process the header parameters
        # process the form parameters
        # process the body parameter
        if asset_ids_dto is not None:
            _body_params = asset_ids_dto

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/octet-stream"]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params["Content-Type"] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(
                ["application/json"]
            )
            if _default_content_type is not None:
                _header_params["Content-Type"] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ["cookie", "api_key", "bearer"]

        return self.api_client.param_serialize(
            method="POST",
            resource_path="/download/archive",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    async def get_download_info(
        self,
        download_info_dto: DownloadInfoDto,
        key: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> DownloadResponseDto:
        """Retrieve download information

        Retrieve information about how to request a download for the specified assets or album. The response includes groups of assets that can be downloaded together.

        :param download_info_dto: (required)
        :type download_info_dto: DownloadInfoDto
        :param key:
        :type key: str
        :param slug:
        :type slug: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_download_info_serialize(
            download_info_dto=download_info_dto,
            key=key,
            slug=slug,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "201": "DownloadResponseDto",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def get_download_info_with_http_info(
        self,
        download_info_dto: DownloadInfoDto,
        key: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[DownloadResponseDto]:
        """Retrieve download information

        Retrieve information about how to request a download for the specified assets or album. The response includes groups of assets that can be downloaded together.

        :param download_info_dto: (required)
        :type download_info_dto: DownloadInfoDto
        :param key:
        :type key: str
        :param slug:
        :type slug: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_download_info_serialize(
            download_info_dto=download_info_dto,
            key=key,
            slug=slug,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "201": "DownloadResponseDto",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    async def get_download_info_without_preload_content(
        self,
        download_info_dto: DownloadInfoDto,
        key: Optional[StrictStr] = None,
        slug: Optional[StrictStr] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Retrieve download information

        Retrieve information about how to request a download for the specified assets or album. The response includes groups of assets that can be downloaded together.

        :param download_info_dto: (required)
        :type download_info_dto: DownloadInfoDto
        :param key:
        :type key: str
        :param slug:
        :type slug: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_download_info_serialize(
            download_info_dto=download_info_dto,
            key=key,
            slug=slug,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "201": "DownloadResponseDto",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _get_download_info_serialize(
        self,
        download_info_dto,
        key,
        slug,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if key is not None:
            _query_params.append(("key", key))

        if slug is not None:
            _query_params.append(("slug", slug))

        # process the header parameters
        # process the form parameters
        # process the body parameter
        if download_info_dto is not None:
            _body_params = download_info_dto

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params["Content-Type"] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(
                ["application/json"]
            )
            if _default_content_type is not None:
                _header_params["Content-Type"] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ["cookie", "api_key", "bearer"]

        return self.api_client.param_serialize(
            method="POST",
            resource_path="/download/info",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )



# coding: utf-8

"""
Immich

Immich API

The version of the OpenAPI document: 2.4.1
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class ReactionType(str, Enum):
    """
    ReactionType
    """

    """
    allowed enum values
    """
    COMMENT = "comment"
    LIKE = "like"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ReactionType from a JSON string"""
        return cls(json.loads(json_str))



# coding: utf-8

"""
Immich

Immich API

The version of the OpenAPI document: 2.4.1
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List
from typing_extensions import Annotated
from immich.client.models.clip_config import CLIPConfig
from immich.client.models.duplicate_detection_config import DuplicateDetectionConfig
from immich.client.models.facial_recognition_config import FacialRecognitionConfig
from immich.client.models.machine_learning_availability_checks_dto import (
    MachineLearningAvailabilityChecksDto,
)
from immich.client.models.ocr_config import OcrConfig
from typing import Optional, Set
from typing_extensions import Self


class SystemConfigMachineLearningDto(BaseModel):
    """
    SystemConfigMachineLearningDto
    """  # noqa: E501

    availability_checks: MachineLearningAvailabilityChecksDto = Field(
        alias="availabilityChecks"
    )
    clip: CLIPConfig
    duplicate_detection: DuplicateDetectionConfig = Field(alias="duplicateDetection")
    enabled: StrictBool
    facial_recognition: FacialRecognitionConfig = Field(alias="facialRecognition")
    ocr: OcrConfig
    urls: Annotated[List[StrictStr], Field(min_length=1)]
    __properties: ClassVar[List[str]] = [
        "availabilityChecks",
        "clip",
        "duplicateDetection",
        "enabled",
        "facialRecognition",
        "ocr",
        "urls",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SystemConfigMachineLearningDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of availability_checks
        if self.availability_checks:
            _dict["availabilityChecks"] = self.availability_checks.to_dict()
        # override the default output from pydantic by calling `to_dict()` of clip
        if self.clip:
            _dict["clip"] = self.clip.to_dict()
        # override the default output from pydantic by calling `to_dict()` of duplicate_detection
        if self.duplicate_detection:
            _dict["duplicateDetection"] = self.duplicate_detection.to_dict()
        # override the default output from pydantic by calling `to_dict()` of facial_recognition
        if self.facial_recognition:
            _dict["facialRecognition"] = self.facial_recognition.to_dict()
        # override the default output from pydantic by calling `to_dict()` of ocr
        if self.ocr:
            _dict["ocr"] = self.ocr.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SystemConfigMachineLearningDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "availabilityChecks": MachineLearningAvailabilityChecksDto.from_dict(
                    obj["availabilityChecks"]
                )
                if obj.get("availabilityChecks") is not None
                else None,
                "clip": CLIPConfig.from_dict(obj["clip"])
                if obj.get("clip") is not None
                else None,
                "duplicateDetection": DuplicateDetectionConfig.from_dict(
                    obj["duplicateDetection"]
                )
                if obj.get("duplicateDetection") is not None
                else None,
                "enabled": obj.get("enabled"),
                "facialRecognition": FacialRecognitionConfig.from_dict(
                    obj["facialRecognition"]
                )
                if obj.get("facialRecognition") is not None
                else None,
                "ocr": OcrConfig.from_dict(obj["ocr"])
                if obj.get("ocr") is not None
                else None,
                "urls": obj.get("urls"),
            }
        )
        return _obj



from __future__ import annotations

from email.message import Message
from mimetypes import guess_extension
from pathlib import Path
from typing import Awaitable, Callable, Optional

import logging
import tqdm

from immich.client.rest import RESTResponseType

logger = logging.getLogger(__name__)


def h(name: str, headers: dict[str, str]) -> Optional[str]:
    """
    Get a header value from a dictionary of headers case-insensitive.
    """
    return next((v for k, v in headers.items() if k.lower() == name.lower()), None)


def filename_from_headers(
    headers: dict[str, str] | None, *, fallback_base: str
) -> Optional[str]:
    """
    Derive a filename from response headers.

    Order of precedence:
    - Content-Disposition filename (if present)
      - if it has no extension, append an extension derived from Content-Type (if available)
    - Content-Type derived extension + `fallback_base`

    :param headers: The response headers.
    :param fallback_base: The filename without extension if we cannot derive the original filename from the Content-Disposition header.

    :returns: The derived filename or None if we cannot derive the filename from the headers.
    """
    if not headers:
        return None

    cd_name: Optional[str] = None
    if cd := h("content-disposition", headers):
        msg = Message()
        msg["content-disposition"] = cd
        if filename := msg.get_filename():
            cd_name = Path(filename).name

    ext = (
        guess_extension(ct.partition(";")[0].strip())
        if (ct := h("content-type", headers))
        else None
    )

    # Content-Disposition wins for the base name; Content-Type is the safety net for the extension.
    if cd_name:
        return cd_name if Path(cd_name).suffix or not ext else f"{cd_name}{ext}"

    return f"{fallback_base}{ext}" if ext else None


def resolve_output_filename(
    headers: dict[str, str] | None,
    *,
    name: Optional[str] = None,
    default_base: str,
    default_ext: Optional[str] = None,
) -> str:
    """
    Resolve an output filename by selecting a base name and an extension.

    Base name precedence:
    - user-supplied `name`
    - Content-Disposition filename
    - `default_base`

    Extension precedence:
    - `default_ext` (if provided; overrides any other extension)
    - extension present on the chosen base name (e.g. from Content-Disposition)
    - Content-Type derived extension
    - none

    :param headers: The response headers.
    :param name: The user-supplied name.
    :param default_base: The default base name, used when no user/header filename can be derived.
    :param default_ext: The default extension (must start with `.`), used when you want to force a specific extension (e.g. `.zip`).

    :returns: The resolved filename.
    """
    if default_ext and not default_ext.startswith("."):
        raise ValueError("default_ext must start with '.' (e.g. '.zip')")

    base = default_base
    ext = None

    header_path = None
    if _ := filename_from_headers(headers, fallback_base=default_base):
        header_path = Path(_)

    # Base name
    if name:
        base = Path(name).stem
    elif header_path:
        base = header_path.stem

    # Extension
    if default_ext:
        ext = default_ext
    elif header_path:
        ext = header_path.suffix
    return f"{base}{ext}" if ext else base


async def download_file(
    make_request: Callable[[Optional[dict[str, str]]], Awaitable[RESTResponseType]],
    out_dir: Path,
    resolve_filename: Callable[[dict[str, str]], str],
    *,
    show_progress: bool = True,
    pbar: Optional[tqdm.tqdm] = None,
    resumeable: bool = True,
) -> Path:
    """
    Download a file and show a progress bar. Allow resuming a download.

    :param make_request: A function that makes a request and returns a RESTResponseType. It takes an optional dictionary of headers.
    :param out_dir: Output directory where the downloaded file will be written.
    :param resolve_filename: Callable that derives the filename. It takes the response headers and returns a string.
    :param show_progress: Whether to show a progress bar.
    :param pbar: A tqdm progress bar to use. If not provided, a new one will be created. If provided, show_progress is ignored. Must be closed by the caller.
    :param resumeable: Whether the download can be resumed from an existing partial `.temp` file via HTTP Range requests.
    :return: The path to the downloaded file.
    """
    resp = None
    temp_path = None
    pbar_created = False

    try:
        file_size = 0
        resp = await make_request(None)

        if not out_dir.is_dir():
            raise ValueError(f"out_dir must be a directory, got: {out_dir}")
        out_dir.mkdir(parents=True, exist_ok=True)

        name = resolve_filename(resp.headers)
        out_path = out_dir / name

        total_size = int(h("Content-Length", resp.headers) or 0)
        temp_path = out_path.with_suffix(out_path.suffix + ".temp")

        if out_path.exists():
            logger.info(f"File already exists: {out_path}")
            file_size = out_path.stat().st_size
            if file_size == total_size:
                resp.close()
                return out_path
            else:
                logger.warning(
                    f"File size mismatch for {out_path}: local file is {file_size} bytes, "
                    f"server reports {total_size} bytes. Skipping download for security reasons. "
                    f"Possible reasons: file was modified locally, file corruption, or server file was updated. "
                    f"If you want to re-download, delete the file and rerun the request."
                )
                resp.close()
                return out_path

        resumed = False
        if temp_path.exists():
            file_size = temp_path.stat().st_size

            if file_size < total_size:
                if resumeable:
                    resume_header = {"Range": f"bytes={file_size}-"}
                    resp.close()
                    resp = await make_request(resume_header)
                    if file_size > 0 and resp.status != 206:
                        logger.debug(
                            "Server does not support resuming; restarting download"
                        )
                        resp.close()
                        temp_path.unlink()
                        file_size = 0
                        resp = await make_request(None)
                    else:
                        logger.info("Resuming download")
                        resumed = True
            else:
                logger.debug(f"Deleting file {out_path} and restarting")
                temp_path.unlink()
                file_size = 0
        else:
            logger.info("Starting download")

        if not pbar:
            pbar = tqdm.tqdm(
                total=total_size,
                unit="B",
                unit_scale=True,
                desc=str(out_path),
                disable=not show_progress,
            )
            pbar_created = True

        if resumed and file_size > 0:
            pbar.update(file_size)
        async with resp:
            mode = "ab" if (resumed and file_size > 0) else "wb"
            with temp_path.open(mode) as f:
                async for chunk in resp.content.iter_chunked(1024 * 1024):
                    if not chunk:
                        continue
                    f.write(chunk)
                    pbar.update(len(chunk))

        temp_path.replace(out_path)
        return out_path

    except Exception:
        if temp_path and temp_path.exists():
            temp_path.unlink()
        raise
    finally:
        if resp and not resp.closed:
            resp.close()

        # Only close pbar if we created it
        if pbar_created and pbar:
            pbar.close()



# coding: utf-8

"""
Immich

Immich API

The version of the OpenAPI document: 2.4.1
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    StrictBool,
    StrictFloat,
    StrictInt,
    StrictStr,
)
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from immich.client.models.asset_visibility import AssetVisibility
from typing import Set
from typing_extensions import Self


class TimeBucketAssetResponseDto(BaseModel):
    """
    TimeBucketAssetResponseDto
    """  # noqa: E501

    city: List[Optional[StrictStr]] = Field(
        description="Array of city names extracted from EXIF GPS data"
    )
    country: List[Optional[StrictStr]] = Field(
        description="Array of country names extracted from EXIF GPS data"
    )
    duration: List[Optional[StrictStr]] = Field(
        description="Array of video durations in HH:MM:SS format (null for images)"
    )
    file_created_at: List[StrictStr] = Field(
        description="Array of file creation timestamps in UTC (ISO 8601 format, without timezone)",
        alias="fileCreatedAt",
    )
    id: List[StrictStr] = Field(description="Array of asset IDs in the time bucket")
    is_favorite: List[StrictBool] = Field(
        description="Array indicating whether each asset is favorited",
        alias="isFavorite",
    )
    is_image: List[StrictBool] = Field(
        description="Array indicating whether each asset is an image (false for videos)",
        alias="isImage",
    )
    is_trashed: List[StrictBool] = Field(
        description="Array indicating whether each asset is in the trash",
        alias="isTrashed",
    )
    latitude: Optional[List[Optional[Union[StrictFloat, StrictInt]]]] = Field(
        default=None,
        description="Array of latitude coordinates extracted from EXIF GPS data",
    )
    live_photo_video_id: List[Optional[StrictStr]] = Field(
        description="Array of live photo video asset IDs (null for non-live photos)",
        alias="livePhotoVideoId",
    )
    local_offset_hours: List[Union[StrictFloat, StrictInt]] = Field(
        description="Array of UTC offset hours at the time each photo was taken. Positive values are east of UTC, negative values are west of UTC. Values may be fractional (e.g., 5.5 for +05:30, -9.75 for -09:45). Applying this offset to 'fileCreatedAt' will give you the time the photo was taken from the photographer's perspective.",
        alias="localOffsetHours",
    )
    longitude: Optional[List[Optional[Union[StrictFloat, StrictInt]]]] = Field(
        default=None,
        description="Array of longitude coordinates extracted from EXIF GPS data",
    )
    owner_id: List[StrictStr] = Field(
        description="Array of owner IDs for each asset", alias="ownerId"
    )
    projection_type: List[Optional[StrictStr]] = Field(
        description='Array of projection types for 360° content (e.g., "EQUIRECTANGULAR", "CUBEFACE", "CYLINDRICAL")',
        alias="projectionType",
    )
    ratio: List[Union[StrictFloat, StrictInt]] = Field(
        description="Array of aspect ratios (width/height) for each asset"
    )
    stack: Optional[
        List[Optional[Annotated[List[StrictStr], Field(min_length=2, max_length=2)]]]
    ] = Field(
        default=None,
        description="Array of stack information as [stackId, assetCount] tuples (null for non-stacked assets)",
    )
    thumbhash: List[Optional[StrictStr]] = Field(
        description="Array of BlurHash strings for generating asset previews (base64 encoded)"
    )
    visibility: List[AssetVisibility] = Field(
        description="Array of visibility statuses for each asset (e.g., ARCHIVE, TIMELINE, HIDDEN, LOCKED)"
    )
    __properties: ClassVar[List[str]] = [
        "city",
        "country",
        "duration",
        "fileCreatedAt",
        "id",
        "isFavorite",
        "isImage",
        "isTrashed",
        "latitude",
        "livePhotoVideoId",
        "localOffsetHours",
        "longitude",
        "ownerId",
        "projectionType",
        "ratio",
        "stack",
        "thumbhash",
        "visibility",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of TimeBucketAssetResponseDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TimeBucketAssetResponseDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "city": obj.get("city"),
                "country": obj.get("country"),
                "duration": obj.get("duration"),
                "fileCreatedAt": obj.get("fileCreatedAt"),
                "id": obj.get("id"),
                "isFavorite": obj.get("isFavorite"),
                "isImage": obj.get("isImage"),
                "isTrashed": obj.get("isTrashed"),
                "latitude": obj.get("latitude"),
                "livePhotoVideoId": obj.get("livePhotoVideoId"),
                "localOffsetHours": obj.get("localOffsetHours"),
                "longitude": obj.get("longitude"),
                "ownerId": obj.get("ownerId"),
                "projectionType": obj.get("projectionType"),
                "ratio": obj.get("ratio"),
                "stack": obj.get("stack"),
                "thumbhash": obj.get("thumbhash"),
                "visibility": obj.get("visibility"),
            }
        )
        return _obj



# coding: utf-8

"""
Immich

Immich API

The version of the OpenAPI document: 2.4.1
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class ManualJobName(str, Enum):
    """
    ManualJobName
    """

    """
    allowed enum values
    """
    PERSON_MINUS_CLEANUP = "person-cleanup"
    TAG_MINUS_CLEANUP = "tag-cleanup"
    USER_MINUS_CLEANUP = "user-cleanup"
    MEMORY_MINUS_CLEANUP = "memory-cleanup"
    MEMORY_MINUS_CREATE = "memory-create"
    BACKUP_MINUS_DATABASE = "backup-database"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ManualJobName from a JSON string"""
        return cls(json.loads(json_str))



# coding: utf-8

"""
Immich

Immich API

The version of the OpenAPI document: 2.4.1
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class SyncRequestType(str, Enum):
    """
    SyncRequestType
    """

    """
    allowed enum values
    """
    ALBUMSV1 = "AlbumsV1"
    ALBUMUSERSV1 = "AlbumUsersV1"
    ALBUMTOASSETSV1 = "AlbumToAssetsV1"
    ALBUMASSETSV1 = "AlbumAssetsV1"
    ALBUMASSETEXIFSV1 = "AlbumAssetExifsV1"
    ASSETSV1 = "AssetsV1"
    ASSETEXIFSV1 = "AssetExifsV1"
    ASSETMETADATAV1 = "AssetMetadataV1"
    AUTHUSERSV1 = "AuthUsersV1"
    MEMORIESV1 = "MemoriesV1"
    MEMORYTOASSETSV1 = "MemoryToAssetsV1"
    PARTNERSV1 = "PartnersV1"
    PARTNERASSETSV1 = "PartnerAssetsV1"
    PARTNERASSETEXIFSV1 = "PartnerAssetExifsV1"
    PARTNERSTACKSV1 = "PartnerStacksV1"
    STACKSV1 = "StacksV1"
    USERSV1 = "UsersV1"
    PEOPLEV1 = "PeopleV1"
    ASSETFACESV1 = "AssetFacesV1"
    USERMETADATAV1 = "UserMetadataV1"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of SyncRequestType from a JSON string"""
        return cls(json.loads(json_str))



# coding: utf-8

"""
Immich

Immich API

The version of the OpenAPI document: 2.4.1
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class QueueName(str, Enum):
    """
    QueueName
    """

    """
    allowed enum values
    """
    THUMBNAILGENERATION = "thumbnailGeneration"
    METADATAEXTRACTION = "metadataExtraction"
    VIDEOCONVERSION = "videoConversion"
    FACEDETECTION = "faceDetection"
    FACIALRECOGNITION = "facialRecognition"
    SMARTSEARCH = "smartSearch"
    DUPLICATEDETECTION = "duplicateDetection"
    BACKGROUNDTASK = "backgroundTask"
    STORAGETEMPLATEMIGRATION = "storageTemplateMigration"
    MIGRATION = "migration"
    SEARCH = "search"
    SIDECAR = "sidecar"
    LIBRARY = "library"
    NOTIFICATIONS = "notifications"
    BACKUPDATABASE = "backupDatabase"
    OCR = "ocr"
    WORKFLOW = "workflow"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of QueueName from a JSON string"""
        return cls(json.loads(json_str))



from immich.client_wrapper.assets_api_wrapped import (
    AssetsApiWrapped as AssetsApiWrapped,
)
from immich.client_wrapper.download_api_wrapped import (
    DownloadApiWrapped as DownloadApiWrapped,
)
from immich.client_wrapper.users_api_wrapped import (
    UsersApiWrapped as UsersApiWrapped,
)

__all__ = [
    "AssetsApiWrapped",
    "DownloadApiWrapped",
    "UsersApiWrapped",
]
